"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var path = _interopRequireWildcard(require("path"));
var _isCi = _interopRequireDefault(require("is-ci"));
var _actionPreflightCheck = _interopRequireDefault(require("./utils/actionPreflightCheck"));
var _resolveAsBin = require("./utils/resolveAsBin");
var _getModularRoot = _interopRequireDefault(require("./utils/getModularRoot"));
var _execAsync = _interopRequireDefault(require("./utils/execAsync"));
var _gitActions = require("./utils/gitActions");
var logger = _interopRequireWildcard(require("./utils/logger"));
var _utils = require("./test/utils");
var _getAllWorkspaces = require("./utils/getAllWorkspaces");
var _selectWorkspaces = require("./utils/selectWorkspaces");
var _unobtrusiveModular = require("./utils/unobtrusiveModular");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }
function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }
async function lint(options, userRegexes = []) {
  var _lintPackageRegexes;
  const {
    all = false,
    fix = false,
    staged = false,
    packages = [],
    ancestors = false,
    descendants = false,
    changed = false,
    includeNonModular = false,
    compareBranch
  } = options;
  const modularRoot = (0, _getModularRoot.default)();
  const lintExtensions = ['.ts', '.tsx', '.js', '.jsx'];
  const [workspaceMap] = await (0, _getAllWorkspaces.getAllWorkspaces)();
  const isSelective = (packages === null || packages === void 0 ? void 0 : packages.length) || ancestors || descendants || changed || compareBranch;
  let selectedTargets;
  if (!isSelective) {
    selectedTargets = [...workspaceMap.keys()];
  } else {
    // Otherwise, calculate which packages are selected
    selectedTargets = await (0, _selectWorkspaces.selectWorkspaces)({
      targets: packages,
      changed,
      ancestors,
      descendants,
      compareBranch
    });
  }

  // Split packages into modular and non-modular testable. Make sure that "root" is not there.
  const [modularTargets, nonModularTargets] = (0, _unobtrusiveModular.partitionPackages)(selectedTargets, workspaceMap, 'lint');

  // Compute patterns to pass Jest for the packages that we want to test
  const packageRegexes = await (0, _unobtrusiveModular.computeRegexesFromPackageNames)(modularTargets);

  // Only set regexes if all or selective options have been specified
  let lintPackageRegexes = isSelective || all ? packageRegexes.map(regex => `<rootDir>/${regex}/src/**/*.{js,jsx,ts,tsx}`) : [];
  if (all && isSelective) {
    logger.warn('You specified --all already; selective options are redundant.');
  }
  if (staged && isSelective) {
    logger.error("Can't specify --staged along with selective options");
    process.exit(-1);
  }

  // Not selective, not --all and no regexes; calculate the file regexes of --diff or --staged
  if (!all && !isSelective && (!_isCi.default || staged) && userRegexes.length === 0) {
    let diffedFiles;
    try {
      // This can throw in case there is no git directory; in this case, we need to continue like nothing happened
      diffedFiles = staged ? (0, _gitActions.getStagedFiles)() : (0, _gitActions.getDiffedFiles)();
    } catch {
      diffedFiles = null;
      logger.log('Getting staged or diffed files failed - are you sure this is a git repo? Falling back to `--all`.');
    }
    if (diffedFiles !== null) {
      if (diffedFiles.length === 0) {
        logger.log('No diffed files detected. Use the `--all` option to lint the entire codebase');
      } else {
        lintPackageRegexes = diffedFiles.filter(p => lintExtensions.includes(path.extname(p))).map(p => `<rootDir>/${p}`);

        // if none of the diffed files meet the extension criteria, do not lint
        // end the process early with a success
        if (!lintPackageRegexes.length) {
          logger.warn('No diffed target files to lint found');
        }
      }
    }
  }

  // If we computed no regexes and there are no non-modular packages to lint, bail out
  if (!((_lintPackageRegexes = lintPackageRegexes) !== null && _lintPackageRegexes !== void 0 && _lintPackageRegexes.length) && !userRegexes && !nonModularTargets.length) {
    process.stdout.write('No workspaces found in selection\n');
    process.exit(0);
  }
  const jestEslintConfig = {
    runner: require.resolve('modular-scripts/jest-runner-eslint'),
    displayName: 'lint',
    rootDir: modularRoot,
    testMatch: lintPackageRegexes,
    testPathIgnorePatterns: ['/node_modules/', '/dist/']
  };
  const lintArgs = [...userRegexes, '--config', (0, _utils.generateJestConfig)(jestEslintConfig)];
  const lintBin = await (0, _resolveAsBin.resolveAsBin)('jest-cli');
  logger.debug(`Regexes for Modular packages being linted: ${JSON.stringify(lintPackageRegexes)}`);
  logger.debug(`User provided regexes: ${JSON.stringify(userRegexes)}`);

  // Lint modular packages
  if (lintPackageRegexes.length || userRegexes.length) {
    try {
      logger.debug(`Running ${lintBin} with cwd: ${(0, _getModularRoot.default)()} and args: ${JSON.stringify(lintArgs)}`);
      await (0, _execAsync.default)(lintBin, lintArgs, {
        cwd: modularRoot,
        log: false,
        // @ts-ignore
        env: {
          MODULAR_ROOT: modularRoot,
          MODULAR_LINT_FIX: String(fix)
        }
      });
      if (staged && fix) {
        lintPackageRegexes = lintPackageRegexes.map(p => p.replace('<rootDir>/', ''));
        (0, _gitActions.addFiles)(lintPackageRegexes);
      }
    } catch (err) {
      logger.debug(err.message);
      // ✕ Modular lint did not pass
      throw new Error('\u2715 Modular lint did not pass');
    }
  }

  // Lint non-modular packages - TODO: Remove conditional & flag in next major release of Modular (5.0.0)
  if (includeNonModular) {
    try {
      logger.debug(`Running lint command in the following non-modular packages: ${JSON.stringify(nonModularTargets)}`);
      for (const target of nonModularTargets) {
        await (0, _execAsync.default)(`yarn`, ['workspace', target, 'lint'], {
          cwd: (0, _getModularRoot.default)(),
          log: false
        });
      }
    } catch (err) {
      logger.debug(err.message);
      // ✕ Modular lint did not pass
      throw new Error('\u2715 Modular lint did not pass');
    }
  }
}
var _default = (0, _actionPreflightCheck.default)(lint);
exports.default = _default;
//# sourceMappingURL=lint.js.map